{ Output files
 (C) 2005-2012, Andreas Berg. All Rights Reserved. }
unit _output;

interface

uses
 _common;

type
 TOutput = record
    Name           : string;
    IdentName      : string;
    FileExt        : string;
    Header         : TDefValue;
    BlockSize      : TAsmValue;
    Limit          : TAsmValue;
    FillSize       : TAsmValue;
    RoundSize      : TAsmValue;
    FillPatternOfs : TDefValue;
    FillPatternLen : TAsmValue;
 end;

var
 OutputList        : array of TOutput;
 OutputCount       : TDefValue;
 CurOutput         : TDefValue;

 OutputMem         : array of byte;
 OutputLen         : TAsmValue;
 WrittenBytes      : TAsmValue;

procedure InitOutputs;
procedure AddOutput(n1, inam1, fExt1: string; hdr1: TDefValue;
 bSiz1, lim1, fil1, rnd1: TAsmValue; fOfs1: TDefValue; fLen1: TAsmValue);

procedure FillRoundOutput(curSize, toBytes: TAsmValue);
procedure WriteOutput;

implementation

uses
 SysUtils, _diskio, _error, _emit, _mem, _main, _func;

procedure InitOutputs;
begin
   SetLength(OutputList, 1);
   OutputCount:= 0;
   CurOutput:= Neg;
   SetLength(OutputMem, 1);
   OutputLen:= 0;
   WrittenBytes:= 0;
end;

procedure AddOutput(n1, inam1, fExt1: string; hdr1: TDefValue;
 bSiz1, lim1, fil1, rnd1: TAsmValue; fOfs1: TDefValue; fLen1: TAsmValue);
var
 oFound            : boolean;
 i                 : TDefValue;
begin
   oFound:= false;
   if OutputCount > 0 then begin
      i:= 0;
      while i < OutputCount do begin
         if OutputList[i].IdentName = inam1 then begin
	    oFound:= true;
            break;
	 end;
         Inc(i);
      end;
   end;
   if not oFound then begin
      SetLength(OutputList, OutputCount + 1);
      OutputList[OutputCount].Name:= n1;
      OutputList[OutputCount].IdentName:= inam1;
      OutputList[OutputCount].FileExt:= fExt1;
      OutputList[OutputCount].Header:= hdr1;
      OutputList[OutputCount].BlockSize:= bSiz1;
      OutputList[OutputCount].Limit:= lim1;
      OutputList[OutputCount].FillSize:= fil1;
      OutputList[OutputCount].RoundSize:= rnd1;
      OutputList[OutputCount].FillPatternOfs:= fOfs1;
      OutputList[OutputCount].FillPatternLen:= fLen1;
      Inc(OutputCount);
      if CurOutput = Neg then CurOutput:= 0;
   end else begin
      ErrorMsg('Output already exist [' + inam1 + ']!');
   end;
end;



procedure FillRoundOutput(curSize, toBytes: TAsmValue);
var
 flOfs, i          : TDefValue;
 flLen, flPos      : TAsmValue;
 b                 : byte;
begin
   flOfs:= OutputList[CurOutput].FillPatternOfs;
   flLen:= OutputList[CurOutput].FillPatternLen;
   flPos:= 0;
   SetLength(OutputMem, toBytes+1);
   i:= curSize;
   while i < toBytes do begin
      b:= 0;
      if flLen > 0 then begin
         b:= MemData[flOfs + flPos];
         Inc(flPos);
         if flPos > (flLen-1) then flPos:= 0;
      end;
      OutputMem[i]:= b;
      Inc(i);
   end;
   OutputLen:= toBytes;
end;

procedure WriteOutput;
var
 fhdl              : file of byte;
begin
   EmitLn('------ START WRITING OUTPUT FILE [' + MakeTime(true) + '] ------');
   if OutputList[CurOutput].FileExt <> '' then begin
      OutFile:= RemoveExt(OutFile) + '.' + OutputList[CurOutput].FileExt;
   end;

   {$I-}
   Assign(fhdl, OutFile);
   Rewrite(fhdl);
      BlockWrite(fhdl, OutputMem[0], OutputLen);
   Close(fhdl);
   {$I+}
   if IOResult = 0 then
    EmitLn('Output Size: ' + IntToStr(OutputLen))
   else
    FatalError('Could not write [' + OutFile + ']!');
   EmitLn('------ END WRITING OUTPUT FILE [' + MakeTime(true) + '] ------');
end;

end.
